% Model file rigid.m 
% This M-file contains the model of the open-loop rigid 
% stick balancer described in Appendix A of "Feedback 
% Control Problems using MATLAB and the Control System 
% Toolbox" by D. K. Frederick and J. H. Chow. 
%
%   states are: x1 = stick angle (rad)
%		x2 = stick angular velocity (rad/s)
%		x3 = cart position  (feet)
%		x4 = cart velocity (ft/s)
%   inputs are: u = voltage applied to motor (volts)
%		w = external force on stick (dimensionless)
%  outputs are: theta = stick angle
%		x = cart position
%
%    state-space equations used:
%	xdot = A * x + B * u + G * w
%	  y  =  C * x

%%%%%%%%%%%%%%%%%%%%%% rigid.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB 
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 5/23/99 HS
%----------------------------------------------------
clear
%----- parameters for 6 ft stick, 2 lb cart,
%	0.2 lb stick, etc.
p = sqrt(8.0);	% 1/s
alpha = 0.15;    % ft/s^2
beta = 10.0;	% seconds
Kp = 0.25;	% 1/ft
Km = 4.64;	% ft/(s^2 V)
%------------------------------------
%------- state-space matrices for rigid stick
A = [  0   1      0       0    ; 
      p^2  0      0   beta*Kp  ; 
       0   0      0       1    ; 
    -alpha  0     0    -beta   ];
B = [  0  ; -Km*Kp ; 0 ; Km ];
C = [1 0 0 0;
     0 0 1 0];
D = zeros(2,1);
G = [ 0 ; 1 ; 0 ; 0 ];
%
%----- transfer function matrix for motor voltage input
[num_mtr,den_mtr] = ss2tf(A,B,C,D);
%---- zero out any small elements to avoid numerical problems
num_mtr = small20(num_mtr,1.0e-10);
%----- transfer function matrix for dist force input
[num_dst,den_dst] = ss2tf(A,G,C,D);
num_dst = small20(num_dst,1.0e-10);
%
%------------------------------------
t = [0:0.01:1]';	% time vector
disp(' ')
disp('t = time vector from 0 to 1.0 seconds')
disp('   Use it when calculating OL responses so the')
disp('   .........stick does not fall too far.')
disp('*******>');pause
condition = -1;	% force menu to appear
while (condition < 1) | (condition > 6),
  disp('  ')
  disp(' 1 <= G(s) motor voltage --> stick angle')
  disp(' 2 <= G(s) motor voltage  --> cart position')
  disp(' 3 <= G(s) disturbance force --> stick angle')
  disp(' 4 <= G(s) disturbance force  --> cart position')
  disp(' 5 <= state-space model')
  disp(' 6 <= create file rigid.mat')
%
   disp('')
  disp('Enter 1...5 to select a model,')
  condition = input('   6 for file, or 0 to terminate => ');
  disp('   ')
%
if condition == 0,
  disp('Terminating rigid stick-balancer model'), 
  format	% reset format to default
  break
end
%-------------- 
if condition == 1,
  disp('G(s): motor voltage --> stick angle')
  numG = num_mtr(1,:)
  denG = den_mtr
  G = tf(numG,denG)
  step(G,t);grid
  title('ol rigid stick: step motor voltage --> stick angle (rad)')
  keyboard
  condition = -1;	% force menu to appear
end	% of condition = 1, so return to menu
%-------------------------------------
%
if condition == 2,
  disp('G(s): motor voltage --> cart position')
  numG = num_mtr(2,:)
  denG = den_mtr
  G = tf(numG,denG)
  step(G,t);grid
  title('ol rigid stick: step motor voltage -->cart position (ft)')
  keyboard
  condition = -1;	% force menu to appear
end	% of condition= 2, so return to menu
%-------------------------------------
%
if condition == 3,
  disp('G(s): disturbance force --> stick angle')
  numG = num_dst(1,:)
  denG = den_dst
  G = tf(numG,denG)
  y = 0.25*impulse(G,t);
  plot(t,y);grid
  title('ol rigid stick: 0.25*impulse dist force --> stick angle (rad)')
  keyboard
  condition = -1;	% force menu to appear
end	% of condition = 3, so return to menu
%-------------------------------------
%
if condition == 4,
  disp('G(s): disturbance force --> cart position')
  numG = num_dst(2,:)
  denG = den_dst
  G = tf(numG,denG)
  y = 0.25*impulse(G,t);
  plot(t,y);grid
  title('ol rigid stick: 0.25*impulse dist force --> cart position (ft)')
  keyboard
  condition = -1;	% force menu to appear
end	% of condition= 4, so return to menu
%-------------------------------------
%
if condition == 5,
  disp('state-space matrices are for:')	
  disp('')	
  disp('   xdot = A * x + B * u + G * w')
  disp('     y  =  x')
  disp('states are: x1 = stick angle (rad)')
  disp('            x2 = stick angular velocity (rad/sec)')
  disp('            x3 = cart position  (feet)')
  disp('            x4 = cart velocity (ft/s)')
  disp(' inputs are: u = voltage applied to motor (volts)')
  disp('             w = external force on stick (dimensionless) ')
  disp('outputs are: theta = stick angle (rad)')
  disp('	     x = cart position (feet)')
  disp('')
  keyboard
  disp('end of state-space form'); disp('  ')
  condition = -1;	% force menu to appear
end	%  of condition = 3, so return to menu
%-------
if condition == 6,
   numG1 = num_mtr(1,:);
   denG1 = den_mtr; 
   numG2 = num_mtr(2,:);
   denG2 = den_mtr;
   numG3 = num_dst(1,:);
   denG3 = den_dst;
   numG4 = num_dst(2,:);
   denG4 = den_dst;
   save rigid.mat numG1 denG1 numG2 denG2 ...
         	numG3 denG3 numG4 denG4 A B C D G t 
  disp('file rigid.mat has been saved')
  disp('*******>');pause
  condition = -1;
end
%------
end	% of while
disp('')
disp('tf form of rigid model in numG & denG if used options 1....4')
disp('State-space rigid model in A, B, C, D if used option 5')
%
%%%%%%%%%%%%%%%%% end of rigid.m %%%%%%%%%%%%%%%%%%%%